@@ -1,25 +1,22 @@
-# coding=utf-8
+import base64
 import copy
 import datetime
 import decimal
 import functools
+from iso8601 import iso8601
 import itertools
+import mimetypes
 import pprint
 import re
 import translationstring
-import warnings
 import types
-
-from iso8601 import iso8601
-
-from .compat import text_, text_type, string_types, xrange, is_nonstr_iter
-
+import warnings
 
 _ = translationstring.TranslationStringFactory('colander')
 
 
-class _required(object):
-    """ Represents a required value in colander-related operations. """
+class _required:
+    """Represents a required value in colander-related operations."""
 
     def __repr__(self):
         return '<colander.required>'
@@ -33,8 +30,8 @@ required = _required()
 _marker = required  # bw compat
 
 
-class _null(object):
-    """ Represents a null value in colander-related operations. """
+class _null:
+    """Represents a null value in colander-related operations."""
 
     def __nonzero__(self):
         return False
@@ -52,7 +49,7 @@ class _null(object):
 null = _null()
 
 
-class _drop(object):
+class _drop:
     """Represents a value that will be dropped from the schema if it
     is missing during *serialization* or *deserialization*.  Passed as
     a value to the `missing` or `default` keyword argument
@@ -183,8 +180,7 @@ class Invalid(Exception):
                 yield tuple(stack)
 
             for child in node.children:
-                for path in traverse(child, stack):
-                    yield path
+                yield from traverse(child, stack)
 
             stack.pop()
 
@@ -237,11 +233,11 @@ class UnsupportedFields(Invalid):
     """
 
     def __init__(self, node, fields, msg=None):
-        super(UnsupportedFields, self).__init__(node, msg)
+        super().__init__(node, msg)
         self.fields = fields
 
 
-class All(object):
+class All:
     """Composite validator which succeeds if none of its
     subvalidators raises an :class:`colander.Invalid` exception"""
 
@@ -257,9 +253,18 @@ class All(object):
                 excs.append(e)
 
         if excs:
-            exc = Invalid(node, [exc.msg for exc in excs])
-            for e in excs:
-                exc.children.extend(e.children)
+            children = []
+            messages = []
+            for exception in excs:
+                if exception.msg is not None:
+                    if is_nonstr_iter(exception.msg):
+                        messages.extend(exception.msg)
+                    else:
+                        messages.append(exception.msg)
+                children.extend(exception.children)
+
+            exc = Invalid(node, messages)
+            exc.children.extend(children)
             raise exc
 
 
@@ -269,7 +274,7 @@ class Any(All):
 
     def __call__(self, node, value):
         try:
-            return super(Any, self).__call__(node, value)
+            return super().__call__(node, value)
         except Invalid as e:
             if len(e.msg) < len(self.validators):
                 # At least one validator did not fail:
@@ -277,25 +282,24 @@ class Any(All):
             raise
 
 
-class Function(object):
+class Function:
     """Validator which accepts a function and an optional message;
     the function is called with the ``value`` during validation.
 
-    If the function returns anything falsy (``None``, ``False``, the
+    If the function returns anything falsey (``None``, ``False``, the
     empty string, ``0``, an object with a ``__nonzero__`` that returns
     ``False``, etc) when called during validation, an
     :exc:`colander.Invalid` exception is raised (validation fails);
     its msg will be the value of the ``msg`` argument passed to this
     class' constructor.
 
-    If the function returns a stringlike object (a ``str`` or
-    ``unicode`` object) that is *not* the empty string , a
-    :exc:`colander.Invalid` exception is raised using the stringlike
-    value returned from the function as the exeption message
+    If the function returns a ``str`` object that is *not* the empty string,
+    a :exc:`colander.Invalid` exception is raised using the string
+    value returned from the function as the exception message
     (validation fails).
 
-    If the function returns anything *except* a stringlike object
-    object which is truthy (e.g. ``True``, the integer ``1``, an
+    If the function returns anything *except* a string object
+    which is truthy (e.g. ``True``, the integer ``1``, an
     object with a ``__nonzero__`` that returns ``True``, etc), an
     :exc:`colander.Invalid` exception is *not* raised (validation
     succeeds).
@@ -331,7 +335,7 @@ class Function(object):
                     self.msg, mapping={'val': value}
                 ),
             )
-        if isinstance(result, string_types):
+        if isinstance(result, str):
             raise Invalid(
                 node,
                 translationstring.TranslationString(
@@ -340,7 +344,7 @@ class Function(object):
             )
 
 
-class Regex(object):
+class Regex:
     """Regular expression validator.
 
     Initialize it with the string regular expression ``regex`` that will
@@ -363,7 +367,7 @@ class Regex(object):
     """
 
     def __init__(self, regex, msg=None, flags=0):
-        if isinstance(regex, string_types):
+        if isinstance(regex, str):
             self.match_object = re.compile(regex, flags)
         else:
             self.match_object = regex
@@ -397,13 +401,71 @@ class Email(Regex):
     """
 
     def __init__(self, msg=None):
-        email_regex = text_(EMAIL_RE)
         if msg is None:
             msg = _("Invalid email address")
-        super(Email, self).__init__(email_regex, msg=msg)
+        super().__init__(EMAIL_RE, msg=msg)
 
 
-class Range(object):
+# Regex for data URLs, loosely based on MDN:
+# https://developer.mozilla.org/en-US/docs/Web/HTTP/Basics_of_HTTP/Data_URLs
+DATA_URL_REGEX = (
+    # data: (required)
+    r'^data:'
+    # optional mime type
+    r'([^;]*)?'
+    # optional base64 identifier
+    r'(;base64)?'
+    # actual data follows the comma
+    r',(.*)$'
+)
+
+
+class DataURL(Regex):
+    """A data URL validator.
+
+    If ``url_msg`` is supplied, it will be the error message to be used when
+    raising :exc:`colander.Invalid` for a syntactically incorrect data URL,
+    defaults to 'Not a data URL'. If, however, the data URL string is
+    syntactically correct but contains an invalid MIME type, ``mimetype_err``
+    is raised (defaults to 'Invalid MIME type'); for incorrectly encoded Base64
+    data, ``base64_err`` is raised (defaults to 'Invalid Base64 encoded data').
+    """
+
+    _URL_ERR = _("Not a data URL")
+    _MIMETYPE_ERR = _("Invalid MIME type")
+    _BASE64_ERR = _("Invalid Base64 encoded data")
+
+    def __init__(
+        self,
+        url_err=_URL_ERR,
+        mimetype_err=_MIMETYPE_ERR,
+        base64_err=_BASE64_ERR,
+    ):
+        self.url_err = url_err
+        self.mimetype_err = mimetype_err
+        self.base64_err = base64_err
+        super().__init__(DATA_URL_REGEX, msg=url_err, flags=re.IGNORECASE)
+
+    def __call__(self, node, value):
+        match_ = self.match_object.match(value)
+        if match_ is None:
+            raise Invalid(node, self.url_err)
+        excs = []
+        mime, is_base64_data, data = match_.groups()
+        if mime and mime not in mimetypes.types_map.values():
+            excs.append(self.mimetype_err)
+        try:
+            if is_base64_data:
+                base64.standard_b64decode(data)
+        except Exception:
+            excs.append(self.base64_err)
+        if len(excs) == 1:
+            raise Invalid(node, excs[0])
+        elif len(excs) == 2:
+            raise Invalid(node, excs)
+
+
+class Range:
     """Validator which succeeds if the value it is passed is greater
     or equal to ``min`` and less than or equal to ``max``.  If ``min``
     is not specified, or is specified as ``None``, no lower bound
@@ -454,7 +516,7 @@ class Range(object):
                 raise Invalid(node, max_err)
 
 
-class Length(object):
+class Length:
     """Validator which succeeds if the value passed to it has a
     length between a minimum and maximum, expressed in the
     optional ``min`` and ``max`` arguments.
@@ -500,7 +562,7 @@ class Length(object):
                 raise Invalid(node, max_err)
 
 
-class OneOf(object):
+class OneOf:
     """Validator which succeeds if the value passed to it is one of
     a fixed set of values"""
 
@@ -517,7 +579,7 @@ class OneOf(object):
             raise Invalid(node, err)
 
 
-class NoneOf(object):
+class NoneOf:
     """Validator which succeeds if the value passed to it is none of a
     fixed set of values.
 
@@ -544,7 +606,7 @@ class NoneOf(object):
         raise Invalid(node, err)
 
 
-class ContainsOnly(object):
+class ContainsOnly:
     """Validator which succeeds if the value passed to is a sequence and each
     element in the sequence is also in the sequence passed as ``choices``.
     This validator is useful when attached to a schemanode with, e.g. a
@@ -607,26 +669,53 @@ def _luhnok(value):
     return checksum
 
 
-# Gingerly lifted from Django 1.3.x:
-# https://github.com/django/django/blob/stable/1.3.x/django/core/validators.py#L45
-# <3 y'all!
-URL_REGEX = (
-    # {http,ftp}s:// (not required)
-    r'^((?:http|ftp)s?://)?'
-    # Domain
-    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+'
-    r'(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'
-    # Localhost
-    r'localhost|'
-    # IPv6 address
-    r'\[[a-f0-9:]+\]|'
-    # IPv4 address
-    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
-    # Optional port
-    r'(?::\d+)?'
-    # Path
-    r'(?:/?|[/?]\S+)$'
-)
+# port from Django 4.1.x:
+# https://github.com/django/django/blob/stable/4.1.x/django/core/validators.py#L70
+
+
+def _make_url_regex_src():
+    ul = "\u00a1-\uffff"  # Unicode letters range (must not be a raw string).
+
+    # IP patterns
+    ipv4_re = (
+        r"(?:0|25[0-5]|2[0-4][0-9]|1[0-9]?[0-9]?|[1-9][0-9]?)"
+        r"(?:\.(?:0|25[0-5]|2[0-4][0-9]|1[0-9]?[0-9]?|[1-9][0-9]?)){3}"
+    )
+    ipv6_re = r"\[[0-9a-f:.]+\]"  # (simple regex, validated later)
+
+    # Host patterns
+    hostname_re = (
+        (r"[a-z" + ul + r"0-9]")
+        + r"(?:"
+        + (r"[a-z" + ul + r"0-9-]{0,61}")
+        + (r"[a-z" + ul + r"0-9]")
+        + r")?"
+    )
+    # Max length for domain name labels is 63 characters per RFC 1034 sec. 3.1
+    domain_re = r"(?:\.(?!-)[a-z" + ul + r"0-9-]{1,63}(?<!-))*"
+    tld_re = (
+        r"\."  # dot
+        r"(?!-)"  # can't start with a dash
+        r"(?:[a-z" + ul + "-]{2,63}"  # domain label
+        r"|xn--[a-z0-9]{1,59})"  # or punycode label
+        r"(?<!-)"  # can't end with a dash
+        r"\.?"  # may have a trailing dot
+    )
+    host_re = "(" + hostname_re + domain_re + tld_re + "|localhost)"
+
+    return (
+        # {http,ftp}s:// (not required)
+        r"^((?:http|ftp)s?://)?"
+        r"(?:[^\s:@/]+(?::[^\s:@/]*)?@)?"  # user:pass authentication
+        r"(?:" + ipv4_re + "|" + ipv6_re + "|" + host_re + ")"
+        r"(?::[0-9]{1,5})?"  # port
+        r"(?:[/?#][^\s]*)?"  # resource path
+        r"\Z"
+    )
+
+
+URL_REGEX = _make_url_regex_src()
+del _make_url_regex_src
 
 url = Regex(URL_REGEX, msg=_('Must be a URL'), flags=re.IGNORECASE)
 
@@ -648,15 +737,15 @@ UUID_REGEX = (
 uuid = Regex(UUID_REGEX, _('Invalid UUID string'), re.IGNORECASE)
 
 
-class SchemaType(object):
-    """ Base class for all schema types """
+class SchemaType:
+    """Base class for all schema types"""
 
     def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
         if listitem:
             selfname = prefix
         else:
-            selfname = '%s%s' % (prefix, node.name)
+            selfname = f'{prefix}{node.name}'
         result[selfname.rstrip('.')] = appstruct
         return result
 
@@ -767,7 +856,7 @@ class Mapping(SchemaType):
             children.append(subval)
         return children
 
-    def _impl(self, node, value, callback):
+    def _impl(self, node, value, callback, default_or_missing):
         value = self._validate(node, value)
 
         error = None
@@ -776,7 +865,10 @@ class Mapping(SchemaType):
         for num, subnode in enumerate(node.children):
             name = subnode.name
             subval = value.pop(name, null)
-            if subval is drop or (subval is null and subnode.default is drop):
+            if subval is drop or (
+                subval is null
+                and getattr(subnode, default_or_missing, None) is drop
+            ):
                 continue
             try:
                 sub_result = callback(subnode, subval)
@@ -815,7 +907,7 @@ class Mapping(SchemaType):
         def callback(subnode, subappstruct):
             return subnode.serialize(subappstruct)
 
-        return self._impl(node, appstruct, callback)
+        return self._impl(node, appstruct, callback, 'default')
 
     def deserialize(self, node, cstruct):
         if cstruct is null:
@@ -824,7 +916,7 @@ class Mapping(SchemaType):
         def callback(subnode, subcstruct):
             return subnode.deserialize(subcstruct)
 
-        return self._impl(node, cstruct, callback)
+        return self._impl(node, cstruct, callback, 'missing')
 
     def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
@@ -832,7 +924,7 @@ class Mapping(SchemaType):
             selfprefix = prefix
         else:
             if node.name:
-                selfprefix = '%s%s.' % (prefix, node.name)
+                selfprefix = f'{prefix}{node.name}.'
             else:
                 selfprefix = prefix
 
@@ -867,7 +959,7 @@ class Mapping(SchemaType):
         return appstruct[path]
 
 
-class Positional(object):
+class Positional:
     """
     Marker abstract base class meaning 'this type has children which
     should be addressed by position instead of name' (e.g. via seq[0],
@@ -970,7 +1062,7 @@ class Tuple(Positional, SchemaType):
         if listitem:
             selfprefix = prefix
         else:
-            selfprefix = '%s%s.' % (prefix, node.name)
+            selfprefix = f'{prefix}{node.name}.'
 
         for num, subnode in enumerate(node.children):
             substruct = appstruct[num]
@@ -992,7 +1084,7 @@ class Tuple(Positional, SchemaType):
             next_name, rest = path.split('.', 1)
         else:
             next_name, rest = path, None
-        for index, next_node in enumerate(node.children):
+        for index, next_node in enumerate(node.children):  # noqa B007
             if next_node.name == next_name:
                 break
         else:
@@ -1011,7 +1103,7 @@ class Tuple(Positional, SchemaType):
             name, rest = path.split('.', 1)
         else:
             name, rest = path, None
-        for index, next_node in enumerate(node.children):
+        for index, next_node in enumerate(node.children):  # noqa B007
             if next_node.name == name:
                 break
         else:
@@ -1128,7 +1220,7 @@ class Sequence(Positional, SchemaType):
         if (
             hasattr(value, '__iter__')
             and not hasattr(value, 'get')
-            and not isinstance(value, string_types)
+            and not isinstance(value, str)
         ):
             return list(value)
         if accept_scalar:
@@ -1143,7 +1235,7 @@ class Sequence(Positional, SchemaType):
             return SequenceItems([])
         return SequenceItems(cstruct)
 
-    def _impl(self, node, value, callback, accept_scalar):
+    def _impl(self, node, value, callback, default_or_missing, accept_scalar):
         if accept_scalar is None:
             accept_scalar = self.accept_scalar
 
@@ -1154,7 +1246,10 @@ class Sequence(Positional, SchemaType):
 
         subnode = node.children[0]
         for num, subval in enumerate(value):
-            if subval is drop or (subval is null and subnode.default is drop):
+            if subval is drop or (
+                subval is null
+                and getattr(subnode, default_or_missing, None) is drop
+            ):
                 continue
             try:
                 sub_result = callback(subnode, subval)
@@ -1198,7 +1293,7 @@ class Sequence(Positional, SchemaType):
         def callback(subnode, subappstruct):
             return subnode.serialize(subappstruct)
 
-        return self._impl(node, appstruct, callback, accept_scalar)
+        return self._impl(node, appstruct, callback, 'default', accept_scalar)
 
     def deserialize(self, node, cstruct, accept_scalar=None):
         """
@@ -1226,19 +1321,19 @@ class Sequence(Positional, SchemaType):
         def callback(subnode, subcstruct):
             return subnode.deserialize(subcstruct)
 
-        return self._impl(node, cstruct, callback, accept_scalar)
+        return self._impl(node, cstruct, callback, 'missing', accept_scalar)
 
     def flatten(self, node, appstruct, prefix='', listitem=False):
         result = {}
         if listitem:
             selfprefix = prefix
         else:
-            selfprefix = '%s%s.' % (prefix, node.name)
+            selfprefix = f'{prefix}{node.name}.'
 
         childnode = node.children[0]
 
         for num, subval in enumerate(appstruct):
-            subname = '%s%s' % (selfprefix, num)
+            subname = f'{selfprefix}{num}'
             subprefix = subname + '.'
             result.update(
                 childnode.typ.flatten(
@@ -1258,13 +1353,13 @@ class Sequence(Positional, SchemaType):
         def rewrite_subpath(subpath):
             if '.' in subpath:
                 suffix = subpath.split('.', 1)[1]
-                return '%s.%s' % (child_name, suffix)
+                return f'{child_name}.{suffix}'
             return child_name
 
         mapstruct = _unflatten_mapping(
             node, paths, fstruct, get_child, rewrite_subpath
         )
-        return [mapstruct[str(index)] for index in xrange(len(mapstruct))]
+        return [mapstruct[str(index)] for index in range(len(mapstruct))]
 
     def set_value(self, node, appstruct, path, value):
         if '.' in path:
@@ -1293,7 +1388,10 @@ Seq = Sequence
 
 
 class String(SchemaType):
-    """A type representing a Unicode string.
+    """A type representing a text string.
+
+    It is always an error to deserialize a non-text/binary type. Binary types
+    are only accepted if the ``encoding`` argument is specified.
 
     This type constructor accepts two arguments:
 
@@ -1304,46 +1402,32 @@ class String(SchemaType):
        this type will not do any special encoding of the appstruct it is
        provided, nor will the ``deserialize`` method of this type do
        any special decoding of the cstruct it is provided; inputs and
-       outputs will be assumed to be Unicode.  ``encoding`` defaults
+       outputs will be assumed to be text.  ``encoding`` defaults
        to ``None``.
 
        If ``encoding`` is ``None``:
 
-       - A Unicode input value to ``serialize`` is returned untouched.
+       - Any value to ``serialize`` is run through the ``str`` function to
+         convert to text, and the result is returned.
 
-       - A non-Unicode input value to ``serialize`` is run through the
-         ``unicode()`` function without an ``encoding`` parameter
-         (``unicode(value)``) and the result is returned.
-
-       - A Unicode input value to ``deserialize`` is returned untouched.
-
-       - A non-Unicode input value to ``deserialize`` is run through the
-         ``unicode()`` function without an ``encoding`` parameter
-         (``unicode(value)``) and the result is returned.
+       - A text input value to ``deserialize`` is returned untouched.
 
        If ``encoding`` is not ``None``:
 
-       - A Unicode input value to ``serialize`` is run through the
-         ``unicode`` function with the encoding parameter
-         (``unicode(value, encoding)``) and the result (a ``str``
-         object) is returned.
+       - Any value to ``serialize`` is run through the ``str`` function to
+         convert to text. The value is then encoded to binary with
+         the encoding parameter (``bytes(value, encoding)``) and the result
+         (a ``bytes`` object) is returned.
 
-       - A non-Unicode input value to ``serialize`` is converted to a
-         Unicode using the encoding (``unicode(value, encoding)``);
-         subsequently the Unicode object is re-encoded to a ``str``
-         object using the encoding and returned.
+       - A text input value to ``deserialize`` is returned untouched.
 
-       - A Unicode input value to ``deserialize`` is returned
-         untouched.
+       - A binary input value to ``deserialize`` is decoded to text using
+         the encoding parameter (``str(value, encoding)``) and the result is
+         returned.
 
-       - A non-Unicode input value to ``deserialize`` is converted to
-         a ``str`` object using ``str(value``).  The resulting str
-         value is converted to Unicode using the encoding
-         (``unicode(value, encoding)``) and the result is returned.
-
-       A corollary: If a string (as opposed to a unicode object) is
+       A corollary: If a ``bytes`` (as opposed to a ``str`` object) is
        provided as a value to either the serialize or deserialize
-       method of this type, and the type also has an non-None
+       method of this type, and the type also has an non-``None``
        ``encoding``, the string must be encoded with the type's
        encoding.  If this is not true, an :exc:`colander.Invalid`
        error will result.
@@ -1366,16 +1450,11 @@ class String(SchemaType):
             return null
 
         try:
-            if isinstance(appstruct, (text_type, bytes)):
-                encoding = self.encoding
-                if encoding:
-                    result = text_(appstruct, encoding).encode(encoding)
-                else:
-                    result = text_type(appstruct)
-            else:
-                result = text_type(appstruct)
-                if self.encoding:
-                    result = result.encode(self.encoding)
+            result = appstruct
+            if not isinstance(result, str):
+                result = str(result)
+            if self.encoding:
+                result = result.encode(self.encoding)
             return result
         except Exception as e:
             raise Invalid(
@@ -1388,20 +1467,17 @@ class String(SchemaType):
 
     def deserialize(self, node, cstruct):
         if cstruct == '' and self.allow_empty:
-            return text_type('')
+            return ''
 
         if not cstruct:
             return null
 
         try:
-            result = cstruct
-            if isinstance(result, (text_type, bytes)):
-                if self.encoding:
-                    result = text_(cstruct, self.encoding)
-                else:
-                    result = text_type(cstruct)
-            else:
-                raise Invalid(node)
+            if isinstance(cstruct, str):
+                return cstruct
+            if self.encoding and isinstance(cstruct, bytes):
+                return str(cstruct, self.encoding)
+            raise Invalid(node)
         except Exception as e:
             raise Invalid(
                 node,
@@ -1411,14 +1487,12 @@ class String(SchemaType):
                 ),
             )
 
-        return result
-
 
 Str = String
 
 
 class Number(SchemaType):
-    """ Abstract base class for float, int, decimal """
+    """Abstract base class for float, int, decimal"""
 
     num = None
 
@@ -1472,7 +1546,7 @@ class Integer(Number):
 
             self.num = _strict_int
 
-        super(Integer, self).__init__()
+        super().__init__()
 
 
 Int = Integer
@@ -1550,7 +1624,7 @@ class Money(Decimal):
     """
 
     def __init__(self):
-        super(Money, self).__init__(decimal.Decimal('.01'), decimal.ROUND_UP)
+        super().__init__(decimal.Decimal('.01'), decimal.ROUND_UP)
 
 
 class Boolean(SchemaType):
@@ -1693,9 +1767,7 @@ class GlobalObject(SchemaType):
         self.package = package
 
     def _pkg_resources_style(self, node, value):
-        """ package.module:attr style """
-        import pkg_resources
-
+        """package.module:attr style"""
         if value.startswith('.') or value.startswith(':'):
             if not self.package:
                 raise Invalid(
@@ -1709,10 +1781,11 @@ class GlobalObject(SchemaType):
                 value = self.package.__name__
             else:
                 value = self.package.__name__ + value
-        return pkg_resources.EntryPoint.parse('x=%s' % value).resolve()
+        value = value.replace(':', '.', 1)
+        return self._zope_dottedname_style(node, value)
 
     def _zope_dottedname_style(self, node, value):
-        """ package.module.attr style """
+        """package.module.attr style"""
         module = self.package and self.package.__name__ or None
         if value == '.':
             if self.package is None:
@@ -1774,7 +1847,7 @@ class GlobalObject(SchemaType):
         if not cstruct:
             return null
 
-        if not isinstance(cstruct, string_types):
+        if not isinstance(cstruct, str):
             raise Invalid(
                 node, _('"${val}" is not a string', mapping={'val': cstruct})
             )
@@ -2131,7 +2204,7 @@ def _add_node_children(node, children):
         _add_node_child(node, n)
 
 
-class _SchemaNode(object):
+class _SchemaNode:
     """
     Fundamental building block of schemas.
 
@@ -2352,7 +2425,7 @@ class _SchemaNode(object):
 
         if self.preparer is not None:
             # if the preparer is a function, call a single preparer
-            if hasattr(self.preparer, '__call__'):
+            if callable(self.preparer):
                 appstruct = self.preparer(appstruct)
             # if the preparer is a list, call each separate preparer
             elif is_nonstr_iter(self.preparer):
@@ -2386,7 +2459,7 @@ class _SchemaNode(object):
         return appstruct
 
     def add(self, node):
-        """ Append a subnode to this node. ``node`` must be a SchemaNode."""
+        """Append a subnode to this node. ``node`` must be a SchemaNode."""
         self.children.append(node)
 
     def insert(self, index, node):
@@ -2469,14 +2542,14 @@ class _SchemaNode(object):
         return cstruct_children(self, cstruct)
 
     def __delitem__(self, name):
-        """ Remove a subnode by name """
+        """Remove a subnode by name"""
         for idx, node in enumerate(self.children[:]):
             if node.name == name:
                 return self.children.pop(idx)
         raise KeyError(name)
 
     def __getitem__(self, name):
-        """ Get a subnode by name. """
+        """Get a subnode by name."""
         val = self.get(name, _marker)
         if val is _marker:
             raise KeyError(name)
@@ -2496,11 +2569,11 @@ class _SchemaNode(object):
         self.add(newnode)
 
     def __iter__(self):
-        """ Iterate over the children nodes of this schema node """
+        """Iterate over the children nodes of this schema node"""
         return iter(self.children)
 
     def __contains__(self, name):
-        """ Return True if subnode named ``name`` exists in this node """
+        """Return True if subnode named ``name`` exists in this node"""
         return self.get(name, _marker) is not _marker
 
     def __repr__(self):
@@ -2594,16 +2667,12 @@ class SequenceSchema(SchemaNode):
         return cloned
 
 
-class deferred(object):
+class deferred:
     """A decorator which can be used to define deferred schema values
     (missing values, widgets, validators, etc.)"""
 
     def __init__(self, wrapped):
-        try:
-            functools.update_wrapper(self, wrapped)
-        except AttributeError:
-            # non-function (raises in Python 2)
-            self.__doc__ = getattr(wrapped, '__doc__', None)
+        functools.update_wrapper(self, wrapped)
         self.wrapped = wrapped
 
     def __call__(self, node, kw):
@@ -2662,7 +2731,7 @@ def _unflatten_mapping(
     return appstruct
 
 
-class instantiate(object):
+class instantiate:
     """
     A decorator which can be used to instantiate :class:`SchemaNode`
     elements inline within a class definition.
@@ -2676,3 +2745,9 @@ class instantiate(object):
 
     def __call__(self, class_):
         return class_(*self.args, **self.kw)
+
+
+def is_nonstr_iter(v):
+    if isinstance(v, str):
+        return False
+    return hasattr(v, '__iter__')
